@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2019 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -656,7 +656,7 @@ static void process_additional_data(xmlNodePtr parent, idmef_additional_data_t *
                 return;
         }
 
-        new = xmlNewTextChild(parent, NULL, (const xmlChar *) "AdditionalData", (const xmlChar *) prelude_string_get_string(out));
+        new = xmlNewChild(parent, NULL, (const xmlChar *) "AdditionalData", NULL);
         if ( ! new ) {
                 prelude_string_destroy(out);
                 return;
@@ -665,6 +665,10 @@ static void process_additional_data(xmlNodePtr parent, idmef_additional_data_t *
         idmef_attr_enum(new, "type", idmef_additional_data_get_type(ad), idmef_additional_data_type_to_string);
         idmef_attr_string(new, "meaning", idmef_additional_data_get_meaning(ad));
 
+        xmlNewTextChild(new, NULL, (const xmlChar *) idmef_additional_data_type_to_string(
+                                idmef_additional_data_get_type(ad)),
+                        (const xmlChar *) prelude_string_get_string(out));
+
         prelude_string_destroy(out);
 }
 
