@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2019 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude-Manager program.
@@ -97,9 +97,9 @@ static int write_all_client(reverse_relay_receiver_t *rrr, prelude_msg_t *msg, u
 {
         int i = 0, ret;
         sensor_fd_t *client;
-        prelude_list_t *tmp;
+        prelude_list_t *tmp, *bkp;
 
-        prelude_list_for_each(sensor_server_get_list(rrr->analyzerid), tmp) {
+        prelude_list_for_each_safe(sensor_server_get_list(rrr->analyzerid), tmp, bkp) {
                 client = prelude_list_entry(tmp, sensor_fd_t, list);
 
                 if ( client->ident != rrr->analyzerid )
@@ -112,7 +112,7 @@ static int write_all_client(reverse_relay_receiver_t *rrr, prelude_msg_t *msg, u
 
                 ret = sensor_server_write_client((server_generic_client_t *) client, prelude_msg_ref(msg));
                 if ( ret < 0 && prelude_error_get_code(ret) != PRELUDE_ERROR_EAGAIN )
-                        prelude_msg_destroy(msg);
+                         server_generic_client_close((server_generic_client_t *) client);
         }
 
         return i;
